@@ -5,7 +5,7 @@
 modname = 'cubicweb_jsonschema'
 distname = 'cubicweb-jsonschema'
 
-numversion = (0, 1, 6)
+numversion = (0, 2, 5)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -15,8 +15,9 @@ description = 'JSON Schema for CubicWeb'
 web = 'http://www.cubicweb.org/project/%s' % distname
 
 __depends__ = {
-    'cubicweb': '>= 3.28.2',
-    'jsonschema': None,
+    'cubicweb[pyramid]': '>= 3.24.1',
+    'six': '>= 1.4.0',
+    'iso8601': None,
     'jsl': None,
     'pyramid': '>= 1.10.0',
 }
