@@ -18,6 +18,7 @@
 from collections import OrderedDict
 from functools import partial
 
+import iso8601
 from six import text_type, string_types
 
 import jsl
@@ -40,7 +41,6 @@ from cubicweb_jsonschema import (
     EDITION_ROLE,
     VIEW_ROLE,
     orm_rtype,
-    relinfo_for,
 )
 from cubicweb_jsonschema.views import jsonschema_section
 
@@ -128,21 +128,28 @@ class yams_match(Predicate):
         return score
 
 
-class partial_match_etype(PartialPredicateMixIn, ExpectedValuePredicate):
-    """Return non-zero if selected class's `etype` attribute matches with
-    `etype` keyword argument from context.
+class partial_yams_match(PartialPredicateMixIn, ExpectedValuePredicate):
+    """Return non-zero if selected class's attributes `etype`, `rtype`, `role`
+    and/or `target_types` match with keyword arguments from context.
     """
 
+    match_attributes = ('etype', 'rtype', 'role', 'target_types')
+
     def __init__(self, *expected, **kwargs):
-        super(partial_match_etype, self).__init__((), **kwargs)
+        super(partial_yams_match, self).__init__((), **kwargs)
 
     def complete(self, cls):
-        self.expected = set([cls.etype])
+        self.expected = {}
+        for name in self.match_attributes:
+            if hasattr(cls, name):
+                self.expected[name] = set([getattr(cls, name)])
 
-    def _values_set(self, cls, req, etype=None, **kwargs):
-        if etype is None:
-            return set([])
-        return set([etype])
+    def _values_set(self, cls, req, **kwargs):
+        values = {}
+        for name in self.match_attributes:
+            if name in kwargs:
+                values[name] = kwargs[name]
+        return values
 
 
 def base_fields(etype):
@@ -183,6 +190,11 @@ class ETypeMapper(JSONSchemaMapper):
     def etype(self):
         return self.cw_extra_kwargs['etype']
 
+    @property
+    def entity(self):
+        """Entity bound to this mapper, None for ETypeMapper."""
+        return None
+
     def jsl_document(self, schema_role=None):
         """Return a jsl.Document for bound entity type with fields relations
         registered to appear in the document (through `jsonschema_section`).
@@ -201,12 +213,11 @@ class ETypeMapper(JSONSchemaMapper):
         attrs['Options'] = default_options(self._cw, self.etype)
         return type(str(self.etype), (jsl.Document, ), attrs)
 
-    def values(self, entity, instance):
+    def values(self, instance):
         """Return a dict with deserialized data from `instance` suitable for
         insertion in CubicWeb database.
-
-        For :class:`ETypeMapper, no entity exists yet so `entity` is None.
         """
+        entity = self.entity
         values = {}
         # Deserialize "jsonschema.object" mappers first.
         for mapper in self._object_mappers():
@@ -214,13 +225,17 @@ class ETypeMapper(JSONSchemaMapper):
         # Then Yams relations.
         if entity is None:
             schema_role = CREATION_ROLE
-            entity_ = self._cw.vreg['etypes'].etype_class(self.etype)(self._cw)
         else:
             schema_role = EDITION_ROLE
-            entity_ = entity
-        for rtype, role, target_types in relinfo_for(entity_, schema_role):
+        for rtype, role, target_types in self.relations(schema_role):
             mapper = self._relation_mapper(rtype, role, target_types)
             values.update(mapper.values(entity, instance))
+        if instance:
+            # All properties in "instance" should have been consumed at this
+            # point.
+            eid = entity.eid if entity is not None else None
+            msg = _('unexpected properties: {}').format(', '.join(instance))
+            raise ValidationError(eid, {None: msg})
         return values
 
     def relations(self, schema_role):
@@ -236,7 +251,9 @@ class ETypeMapper(JSONSchemaMapper):
         except KeyError:
             raise ValueError('unhandled schema role "{0}" in {1}'.format(
                 schema_role, self))
-        entity = self._cw.vreg['etypes'].etype_class(self.etype)(self._cw)
+        entity = self.entity
+        if entity is None:
+            entity = self._cw.vreg['etypes'].etype_class(self.etype)(self._cw)
         rsection = self._cw.vreg['uicfg'].select(
             'jsonschema', self._cw, entity=entity)
         return rsection.relations_by_section(entity, 'inlined', permission)
@@ -290,6 +307,7 @@ class EntityMapper(ETypeMapper):
 
     @property
     def entity(self):
+        """Live entity from selection context."""
         return self.cw_extra_kwargs['entity']
 
     @property
@@ -302,14 +320,14 @@ class EntityMapper(ETypeMapper):
             entity=self.entity, rtype=rtype,
             role=role, target_types=target_types)
 
-    def serialize(self, relinfo):
+    def serialize(self):
         """Return the serialized value entity bound to this mapper."""
         entity = self.entity
         entity.complete()
         data = {}
         for mapper in self._object_mappers():
             data[mapper.name] = mapper.serialize(entity)
-        for rtype, role, target_types in relinfo:
+        for rtype, role, target_types in self.relations(VIEW_ROLE):
             relation_mapper = self._relation_mapper(rtype, role, target_types)
             value = relation_mapper.serialize(entity)
             if value is None:
@@ -318,6 +336,14 @@ class EntityMapper(ETypeMapper):
         return data
 
 
+class TargetEntityMapper(EntityMapper, TargetETypeMapper):
+    """Specialized version of :class:`TargetETypeMapper` and
+    :class:`EntityMapper` selectable for an *entity* as target of (`rtype`,
+    `role`) relation.
+    """
+    __select__ = match_kwargs('entity', 'rtype', 'role')
+
+
 class EntityCollectionMapper(JSONSchemaMapper):
     """Mapper for a collection of entities."""
     __regid__ = 'jsonschema.collection'
@@ -436,8 +462,16 @@ class BaseRelationMapper(JSONSchemaMapper):
 
     @cachedproperty
     def description(self):
+        ntargets = len(self.target_types)
+        if ntargets > 1:
+            return None
+        elif ntargets == 1:
+            targettype = self.target_types[0]
+        else:
+            targettype = None
         eschema = self._cw.vreg.schema[self.etype]
-        rdef = eschema.rdef(self.rtype, role=self.role, takefirst=True)
+        rdef = eschema.rdef(self.rtype, role=self.role,
+                            targettype=targettype)
         if rdef.description:
             return self._cw._(rdef.description)
 
@@ -467,20 +501,8 @@ class BaseRelationMapper(JSONSchemaMapper):
 
         If present, the field matching mapped `rtype` is removed from
         `instance` after processing.
-
-        If absent and `entity` is not None, the default value for mapped
-        relation is returned as value of the dictionnary.
         """
-        try:
-            value = instance.pop(self.rtype)
-        except KeyError:
-            if entity is None:
-                return {}
-            rschema = entity.e_schema.rdef(self.rtype, self.role)
-            value = rschema.default
-        else:
-            value = self._type(value)
-        return {self.orm_rtype: value}
+        raise NotImplementedError()
 
     def serialize(self, entity):
         """Return the serialized value for this field."""
@@ -546,6 +568,31 @@ class AttributeMapper(BaseRelationMapper):
                 self.add_constraint(constraint, kwargs)
         return super(AttributeMapper, self).jsl_field(schema_role, **kwargs)
 
+    def values(self, entity, instance):
+        """Return a dictionary holding deserialized value for mapped attribute.
+
+        If mapped attribute is absent from `instance` and `entity` is not
+        None, the default value for attribute is returned as value of the
+        dictionnary.
+        """
+        try:
+            value = instance.pop(self.rtype)
+        except KeyError:
+            if entity is None:
+                return {}
+            rschema = entity.e_schema.rdef(self.rtype, self.role)
+            value = rschema.default
+        else:
+            value = self._type(value)
+            if entity is not None and getattr(entity, self.orm_rtype) == value:
+                # Do not trigger update, if the value has not changed.
+                # This is useful for PUT requests for which all fields should
+                # be present even if they are not changed. By skipping
+                # unchanged value, we avoid possible security check that would
+                # be meaningless since the value is not changed.
+                return {}
+        return {self.orm_rtype: value}
+
     def serialize(self, entity):
         value = getattr(entity, self.orm_rtype)
         if value is not None:
@@ -635,6 +682,11 @@ class DateMapper(StringMapper):
     #:
     format = 'date'
 
+    @staticmethod
+    def _type(value):
+        """Return a datetime object parsed from ISO8601 `value` string."""
+        return iso8601.parse_date(value)
+
 
 class DatetimeMapper(DateMapper):
     """Attribute mapper for Yams' (TZ)Datetime type."""
@@ -668,7 +720,7 @@ class CompoundMapper(JSONSchemaMapper):
     """
     __regid__ = 'jsonschema.object'
     __abstract__ = True
-    __select__ = partial_match_etype()
+    __select__ = partial_yams_match()
 
     #: entity type holding this compound field.
     etype = None
@@ -677,6 +729,9 @@ class CompoundMapper(JSONSchemaMapper):
     #: title of the field
     title = None
 
+    # sequence of 'relation/role' computed from 'relations'
+    _relations = ()
+
     @classproperty
     def name(cls):
         """name of the property to be inserted in the main JSON Schema
@@ -686,25 +741,46 @@ class CompoundMapper(JSONSchemaMapper):
 
     @classmethod
     def __registered__(cls, reg):
+        if not cls.relations:
+            raise ValueError(
+                '{} is missing a "relations" class attribute'.format(cls))
+        # Compute relation/role pairs from relations
+        relations = []
+        for relinfo in cls.relations:
+            try:
+                rtype, role = relinfo
+            except ValueError:
+                rtype = relinfo
+                role = 'subject'
+            relations.append((rtype, role))
+        cls._relations = tuple(relations)
+        # Check for name or relation duplication
         for obj in reg[cls.__regid__]:
             if obj == cls:
                 continue
-            if obj.name == cls.name:
-                # Make sure 'name' is unique amongst objects with
-                # 'jsonschema.object' regid.
-                raise ValueError('a class with name "{}" is already '
-                                 'registered'.format(cls.name))
             if obj.etype == cls.etype:
+                if obj.name == cls.name:
+                    # Make sure 'name' is unique amongst objects with
+                    # 'jsonschema.object' regid.
+                    raise ValueError('a class with name "{}" is already '
+                                     'registered'.format(cls.name))
                 # Prevent duplicate mapping of the same etype/rtype.
-                common_rtypes = set(obj.relations) & set(cls.relations)
+                common_rtypes = set(obj._relations) & set(cls._relations)
                 if common_rtypes:
                     raise ValueError(
                         'duplicate relation mapping for "{}": {}'.format(
-                            cls.etype, ', '.join(common_rtypes))
+                            cls.etype,
+                            ', '.join('-'.join(rel) for rel in common_rtypes),
+                        )
                     )
         # Hide relations mapped to this document from etype JSON Schema.
-        for rtype in cls.relations:
-            jsonschema_section.tag_subject_of((cls.etype, rtype, '*'), 'hidden')
+        for rtype, role in cls._relations:
+            if role == 'object':
+                jsonschema_section.tag_object_of(
+                    ('*', rtype, cls.etype), 'hidden')
+            else:
+                jsonschema_section.tag_subject_of(
+                    (cls.etype, rtype, '*'), 'hidden')
         return super(CompoundMapper, cls).__registered__(reg)
 
     @property
@@ -714,13 +790,13 @@ class CompoundMapper(JSONSchemaMapper):
 
     def jsl_field(self, schema_role):
         attrs = {}
-        for rtype in self.relations:
+        for rtype, role in self._relations:
             rschema = self._cw.vreg.schema[rtype]
             target_types = {
-                t.type for t in rschema.targets(self.etype, 'subject')}
+                t.type for t in rschema.targets(self.etype, role)}
             mapper = self._cw.vreg['mappers'].select(
                 'jsonschema.relation', self._cw,
-                etype=self.etype, rtype=rtype, role='subject',
+                etype=self.etype, rtype=rtype, role=role,
                 target_types=target_types,
             )
             attrs[mapper.orm_rtype] = mapper.jsl_field(schema_role)
@@ -739,8 +815,8 @@ class CompoundMapper(JSONSchemaMapper):
             # Continue with an empty "value" that would get filled by compound
             # relation mappers with default or None values.
             values = {}
-        for rtype in self.relations:
-            mapper = self._relation_mapper(rtype)
+        for rtype, role in self._relations:
+            mapper = self._relation_mapper(rtype, role)
             values.update(
                 mapper.values(entity, values))
         return values
@@ -749,20 +825,20 @@ class CompoundMapper(JSONSchemaMapper):
         assert entity.cw_etype == self.etype, \
             'cannot serialize {} with {}'.format(entity, self)
         data = {}
-        for rtype in self.relations:
-            mapper = self._relation_mapper(rtype)
+        for rtype, role in self._relations:
+            mapper = self._relation_mapper(rtype, role)
             value = mapper.serialize(entity)
             if value is not None:
                 data[mapper.orm_rtype] = value
         return data
 
-    def _relation_mapper(self, rtype):
+    def _relation_mapper(self, rtype, role='subject'):
         vreg = self._cw.vreg
         rschema = vreg.schema[rtype]
-        target_types = {t.type for t in rschema.targets(self.etype, 'subject')}
+        target_types = {t.type for t in rschema.targets(self.etype, role)}
         return vreg['mappers'].select(
             'jsonschema.relation', self._cw, etype=self.etype,
-            rtype=rtype, role='subject', target_types=target_types,
+            rtype=rtype, role=role, target_types=target_types,
         )
 
 
@@ -793,6 +869,8 @@ class _RelationMapper(BaseRelationMapper):
                         'inconsistent cardinalities within {0} relation '
                         'definitions'.format(self.rtype))
                 if cardinality != '*':
+                    if 'required' not in kwargs and cardinality in '1+':
+                        kwargs['required'] = True
                     if 'min_items' not in kwargs:
                         kwargs['min_items'] = 0 if cardinality in '?' else 1
                     if 'max_items' not in kwargs and cardinality in '?1':
@@ -820,7 +898,6 @@ class InlinedRelationMapper(_RelationMapper):
                 target_types={self.etype},
             )
             target_document = mapper.jsl_document(schema_role)
-            assert target_document.get_definition_id() == target_type
             yield jsl.fields.DocumentField(target_document, as_ref=True)
 
     def values(self, entity, instance):
@@ -853,7 +930,7 @@ class InlinedRelationMapper(_RelationMapper):
         )
         result = []
         for subinstance in values:
-            subvalues = target_mapper.values(None, subinstance)
+            subvalues = target_mapper.values(subinstance)
             result.append(self._cw.create_entity(target_type, **subvalues))
         return {self.orm_rtype: result}
 
@@ -863,12 +940,15 @@ class InlinedRelationMapper(_RelationMapper):
         if not rset:
             return None
 
-        def adapt(entity):
-            return self._cw.vreg['adapters'].select(
-                'IJSONSchema', self._cw, entity=entity,
-                rtype=self.rtype, role=neg_role(self.role))
+        def serialize(entity):
+            mapper = self._cw.vreg['mappers'].select(
+                'jsonschema.entity', self._cw, entity=entity,
+                rtype=self.rtype, role=neg_role(self.role),
+                target_types={self.etype},
+            )
+            return mapper.serialize()
 
-        return [adapt(related).serialize() for related in rset.entities()]
+        return [serialize(related) for related in rset.entities()]
 
 
 class ETypeRelationMapper(_RelationMapper):
