@@ -143,7 +143,8 @@ class EntitiesTC(BaseTC):
             cwuser_eid = cnx.find('CWUser', login=u'admin')[0][0]
         self.login()
         url = '/book/%s/author' % book.eid
-        self.webapp.post_json(url, [str(cwuser_eid)], status=400,
+        # TODO: status should be 4xx
+        self.webapp.post_json(url, [str(cwuser_eid)], status=500,
                               headers={'accept': 'application/json'})
 
     def test_put_with_incomplete_data(self):
@@ -227,7 +228,7 @@ class EntitiesTC(BaseTC):
                 cnx, etype='Author').view_schema()
         self.assertEqual(
             res.location,
-            'https://localhost:80/Author/{}'.format(author_eid))
+            'https://localhost:80/author/{}/'.format(author_eid))
         jsonschema.validate(entity, jschema)
         self.assertEqual(entity['name'], u'bob')
         with self.admin_access.cnx() as cnx:
@@ -257,7 +258,7 @@ class EntitiesTC(BaseTC):
                 cnx, etype='Book').view_schema()
         self.assertEqual(
             res.location,
-            'https://localhost:80/Book/{}'.format(book_eid))
+            'https://localhost:80/book/{}/'.format(book_eid))
         jsonschema.validate(entity, jschema)
         self.assertEqual(entity['title'], u'introducing cubicweb-jsonschema')
         with self.admin_access.cnx() as cnx:
@@ -273,30 +274,93 @@ class EntitiesTC(BaseTC):
             u'born': u'1986',
         }
         self.login()
-        res = self.webapp.post_json('/author/', data, status=400,
+        res = self.webapp.post_json('/author/', data, status=422,
                                     headers={'Accept': 'application/json'})
-        errors = res.json_body['errors']
-        # See https://github.com/Julian/jsonschema/issues/243
-        hint = "'born' was unexpected"
-        expected = [{'status': 422,
-                     'details': ("Additional properties are not allowed "
-                                 "(%s)" % hint)}]
-        self.assertCountEqual(errors, expected)
+        self.assertEqual(res.content_type, 'application/problem+json')
+        self.assertEqual(res.json_body, {
+            u'status': 422,
+            u'title': u'Unprocessable Entity',
+            u'detail': u'unexpected properties: born',
+        })
 
     def test_validationerror_nosource(self):
         """Test validation_failed view with no specific source entry."""
+        with self.admin_access.cnx() as cnx:
+            gid = cnx.find('CWGroup', name=u'users')[0][0]
         data = {
             'login': u'bob',
             'upassword': u'pass',
+            'in_group': [str(gid)],
         }
         with patch('cubicweb.req.RequestSessionBase.create_entity',
                    side_effect=ValidationError(None, {None: u'unmapped'})):
             self.login()
-            res = self.webapp.post_json('/cwuser/', data, status=400,
+            res = self.webapp.post_json('/cwuser/', data, status=422,
                                         headers={'Accept': 'application/json'})
-            errors = res.json_body['errors']
-            expected = [{'status': 422, 'details': 'unmapped'}]
-            self.assertCountEqual(errors, expected)
+            self.assertEqual(res.content_type, 'application/problem+json')
+            self.assertEqual(res.json_body, {
+                u'status': 422,
+                u'title': u'Unprocessable Entity',
+                u'detail': u'unmapped',
+            })
+
+    def test_validationerror_integrityerror(self):
+        with self.admin_access.cnx() as cnx:
+            gid = cnx.find('CWGroup', name=u'users')[0][0]
+            self.create_user(cnx, u'bob')
+            cnx.commit()
+        data = {
+            'login': 'bob',
+            'upassword': 'pass',
+            'in_group': [str(gid)],
+        }
+        self.login()
+        res = self.webapp.post_json('/cwuser/', data, status=422,
+                                    headers={'Accept': 'application/json'})
+        self.assertEqual(res.content_type, 'application/problem+json')
+        self.assertEqual(res.json_body, {
+            u'status': 422,
+            u'title': u'Unprocessable Entity',
+            u'detail': u'some relations violate a unicity constraint',
+            u'invalid-params': [
+                {u'name': u'login',
+                 u'reason': u'login is part of violated unicity constraint'}],
+        })
+
+    def test_exception(self):
+        with self.admin_access.cnx() as cnx:
+            gid = cnx.find('CWGroup', name=u'users')[0][0]
+        data = {
+            'login': 'bob',
+            'upassword': 'pass',
+            'in_group': [str(gid)],
+        }
+        self.login()
+        error = ValueError("invalid literal for int() with base 10: 'foo'")
+        with patch('cubicweb.req.RequestSessionBase.create_entity',
+                   side_effect=error):
+            res = self.webapp.post_json('/cwuser/', data, status=500,
+                                        headers={'Accept': 'application/json'})
+            self.assertEqual(res.content_type, 'application/problem+json')
+            self.assertEqual(res.json_body, {
+                u'status': 500,
+                u'title': u'Internal Server Error',
+                u'detail': u"invalid literal for int() with base 10: 'foo'",
+            })
+
+    def test_exception_no_detail(self):
+        data = {'title': 'thing'}
+        self.login()
+
+        with patch('cubicweb.req.RequestSessionBase.create_entity',
+                   side_effect=NotImplementedError()):
+            res = self.webapp.post_json('/book/', data, status=500,
+                                        headers={'Accept': 'application/json'})
+            self.assertEqual(res.content_type, 'application/problem+json')
+            self.assertEqual(res.json_body, {
+                u'status': 500,
+                u'title': u'Internal Server Error',
+            })
 
 
 if __name__ == '__main__':
