@@ -19,49 +19,19 @@
 from base64 import b64encode
 from copy import deepcopy
 
+from six import text_type
+
 from cubicweb import ValidationError
 from cubicweb.devtools.testlib import CubicWebTC
 
-from cubicweb_jsonschema import (
-    VIEW_ROLE,
-)
 from cubicweb_jsonschema.entities.ijsonschema import (
     jsonschema_adapter,
-    jsonschema_validate,
 )
 
 
 class MiscTC(CubicWebTC):
     """Tests for functions and mixins in entities module."""
 
-    def test_relinfo_for(self):
-        expected_fields_name = ['login', 'firstname', 'surname',
-                                'last_login_time', 'in_group', 'use_email',
-                                'picture']
-        with self.admin_access.cnx() as cnx:
-            user = cnx.find('CWUser', login=u'admin').one()
-            adapter = user.cw_adapt_to('IJSONSchema')
-            fields_name = [rtype
-                           for rtype, _, _ in adapter.relinfo_for(VIEW_ROLE)]
-            self.assertCountEqual(fields_name, expected_fields_name)
-            self.create_user(cnx, u'bob')
-            cnx.commit()
-        with self.new_access(u'bob').cnx() as cnx:
-            user = cnx.find('CWUser', login=u'admin').one()
-            adapter = user.cw_adapt_to('IJSONSchema')
-            fields_name = [rtype
-                           for rtype, _, _ in adapter.relinfo_for(VIEW_ROLE)]
-            self.assertCountEqual(fields_name, expected_fields_name)
-
-    def test_relinfo_for_accounts_for_uicfg(self):
-        with self.admin_access.cnx() as cnx:
-            adapter = jsonschema_adapter(cnx, etype='CWUser')
-            fields = dict(
-                (rtype, targets)
-                for rtype, _, targets in adapter.relinfo_for(VIEW_ROLE))
-            # EmailAlias no in, per jsonschema_section.
-            self.assertEqual(fields['use_email'], {'EmailAddress'})
-
     def test_document_tree_nested(self):
         with self.admin_access.cnx() as cnx:
             mapper = cnx.vreg['mappers'].select(
@@ -110,7 +80,7 @@ class IJSONSchemaETypeAdapterTC(CubicWebTC):
         user_creation_schema = creation_schema['properties']
         expected_items = {
             'type': 'string',
-            'oneOf': [{'enum': [str(eid)], 'title': title}
+            'oneOf': [{'enum': [text_type(eid)], 'title': title}
                       for eid, title in groups],
         }
         self.assertEqual(user_creation_schema['in_group']['items'],
@@ -144,15 +114,6 @@ class IJSONSchemaETypeAdapterTC(CubicWebTC):
             j_schema = adapter.creation_schema()
         self.assertIn('Thumbnail', j_schema['definitions'])
 
-    def test_etype_validate(self):
-        with self.admin_access.cnx() as cnx:
-            adapter = jsonschema_adapter(cnx, etype='CWUser')
-            schema = adapter.creation_schema()
-            instance = {'login': 1, 'upassword': '123'}
-            with self.assertRaises(ValidationError) as cm:
-                jsonschema_validate(schema, instance)
-            self.assertIn("1 is not of type 'string'", str(cm.exception))
-
     def test_create_entity(self):
         with self.admin_access.cnx() as cnx:
             group = cnx.find('CWGroup', name=u'users').one()
@@ -161,15 +122,28 @@ class IJSONSchemaETypeAdapterTC(CubicWebTC):
                 u'login': u'bob',
                 u'upassword': u'123',
                 u'firstname': u'Bob',
+                u'in_group': [str(group.eid)],
             }
             user = adapter.create_entity(instance)
-            # Create `in_group` relationship by hand because it is not
-            # included in JSON schema (not inlined).
-            user.cw_set(in_group=group)
             cnx.commit()
             for attrname, value in instance.items():
                 self.assertEqual(getattr(user, attrname), value)
 
+    def test_create_entity_default_required(self):
+        """Not supplying a required property which schema has a "default" flag
+        is allowed.
+        """
+        with self.admin_access.cnx() as cnx:
+            adapter = jsonschema_adapter(cnx, etype='Photo')
+            j_schema = adapter.creation_schema()
+            self.assertIn('media_type', j_schema['required'])
+            instance = {
+                'data': u'plop',
+            }
+            entity = adapter.create_entity(instance)
+            cnx.commit()
+            self.assertEqual(entity.media_type, u'png')
+
     def test_create_entity_inlined(self):
         with self.admin_access.cnx() as cnx:
             adapter = jsonschema_adapter(cnx, etype='Photo')
@@ -204,27 +178,11 @@ class IJSONSchemaETypeAdapterTC(CubicWebTC):
         self.assertNotIn('required', view_schema)
         self.assertIn('properties', view_schema)
         self.assertCountEqual(creation_schema['required'],
-                              ['login', 'upassword'])
+                              ['login', 'upassword', 'in_group'])
 
 
 class IJSONSchemaRelationTargetETypeAdapterTC(CubicWebTC):
 
-    def test_filtered_relinfo(self):
-        with self.admin_access.cnx() as cnx:
-            adapter = jsonschema_adapter(
-                cnx, etype='CWUser', rtype='in_group', role='subject')
-            self.assertIsNotNone(adapter)
-            relinfos = [(rtype, role)
-                        for rtype, role, _ in adapter.relinfo_for('creation')]
-            self.assertNotIn(('in_group', 'subject'), relinfos)
-            creation_schema = adapter.creation_schema()
-            view_schema = adapter.view_schema()
-        creation_props = creation_schema['properties']
-        self.assertTrue(creation_props)  # Ensure it's not empty.
-        self.assertNotIn('in_group', creation_props)
-        view_props = view_schema['properties']
-        self.assertNotIn('in_group', view_props)
-
     def test_creation_set_relation(self):
         with self.admin_access.cnx() as cnx:
             adapter = jsonschema_adapter(
@@ -242,7 +200,7 @@ class IJSONSchemaRelationTargetETypeAdapterTC(CubicWebTC):
                 }
                 adapter.create_entity(instance, group)
                 cnx.commit()
-            self.assertIn("'in_group' was unexpected", str(cm.exception))
+            self.assertIn('unexpected properties: in_group', str(cm.exception))
 
 
 class IJSONSchemaEntityAdapterTC(CubicWebTC):
@@ -255,17 +213,6 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
             j_schema = adapter.edition_schema()
         self.assertNotIn(u'lastname', j_schema['properties'])
 
-    def test_entity_validate(self):
-        with self.admin_access.cnx() as cnx:
-            user = cnx.find('CWUser', login=u'admin').one()
-            adapter = user.cw_adapt_to('IJSONSchema')
-            schema = adapter.edition_schema()
-            instance = {'login': 'bob', 'upassword': 1}
-            with self.assertRaises(ValidationError) as cm:
-                jsonschema_validate(schema, instance, user)
-            self.assertEqual(cm.exception.entity, user)
-            self.assertIn("1 is not of type 'string'", str(cm.exception))
-
     def test_edition_schema_role(self):
         with self.admin_access.cnx() as cnx:
             user = cnx.find('CWUser', login=u'admin').one()
@@ -296,7 +243,7 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
             users = cnx.find('CWGroup', name=u'users').one()
             adapter = jsonschema_adapter(cnx, etype='CWUser')
             instance = {'login': 'bob', 'upassword': 'sponge',
-                        'in_group': [str(users.eid)],
+                        'in_group': [text_type(users.eid)],
                         'use_email': [{'address': 'bob@sponge.com'}]}
             entity = adapter.create_entity(instance)
             self.assertEqual(entity.login, 'bob')
@@ -315,7 +262,7 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
             adapter = jsonschema_adapter(cnx, entity=entity)
 
             instance = {'login': 'bobby',
-                        'in_group': [str(users.eid)],
+                        'in_group': [text_type(users.eid)],
                         'use_email': [{'address': 'bob@sponge.com'}]}
             adapter.edit_entity(instance)
             entity.cw_clear_all_caches()
@@ -329,8 +276,8 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
             self.assertEqual(entity.use_email[0].address, 'bob@sponge.com')
 
             instance = {'login': 'bobby',
-                        'in_group': [str(users.eid),
-                                     str(guests.eid)],
+                        'in_group': [text_type(users.eid),
+                                     text_type(guests.eid)],
                         'use_email': [{'address': 'bobby@sponge.com'}]}
             adapter.edit_entity(instance)
             entity.cw_clear_all_caches()
@@ -342,7 +289,7 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
             self.assertEqual(entity.use_email[0].address, 'bobby@sponge.com')
 
             instance = {'login': 'bobby',
-                        'in_group': [str(users.eid)],
+                        'in_group': [text_type(users.eid)],
                         'use_email': [{'address': 'bobby@sponge.com'},
                                       {'address': 'bob.sponge@sponge.com'}]}
             adapter.edit_entity(instance)
@@ -354,7 +301,7 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
                                   ['bobby@sponge.com', 'bob.sponge@sponge.com'])
 
             instance = {'login': 'bobby',
-                        'in_group': [str(users.eid)],
+                        'in_group': [text_type(users.eid)],
                         'use_email': [{'address': 'bob.sponge@sponge.com'}]}
             adapter.edit_entity(instance)
             entity.cw_clear_all_caches()
@@ -364,7 +311,7 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
 
             entity.cw_set(use_email=cnx.create_entity('EmailAlias'))
             instance = {'login': 'bobby',
-                        'in_group': [str(users.eid)],
+                        'in_group': [text_type(users.eid)],
                         'use_email': []}
             adapter.edit_entity(instance)
             entity.cw_clear_all_caches()
@@ -383,7 +330,7 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
             email.cw_clear_all_caches()
 
             groups = entity.related('in_group', 'subject').entities()
-            group_ids = [str(group.eid) for group in groups]
+            group_ids = [text_type(group.eid) for group in groups]
             expected = {
                 u'firstname': u'Bob',
                 u'login': u'bob',
@@ -394,23 +341,6 @@ class IJSONSchemaEntityAdapterTC(CubicWebTC):
             self.assertEqual(data, expected)
 
 
-class IJSONSchemaRelatedEntityAdapterTC(CubicWebTC):
-
-    def test_filtered_relinfo(self):
-        with self.admin_access.cnx() as cnx:
-            email = cnx.create_entity(
-                'EmailAddress', address=u'bob@sponge.com')
-            self.create_user(cnx, u'bob', use_email=email)
-            cnx.commit()
-            adapter = jsonschema_adapter(
-                cnx, entity=email, rtype='use_email', role='object')
-            self.assertIsNotNone(adapter)
-            relinfos = [(rtype, role)
-                        for rtype, role, _ in adapter.relinfo_for('edition')]
-            expected = [('alias', 'subject'), ('address', 'subject')]
-            self.assertCountEqual(relinfos, expected)
-
-
 if __name__ == '__main__':
     import unittest
     unittest.main()
