@@ -15,13 +15,16 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 import base64
+from datetime import datetime
 import unittest
 
 import jsl
 import jsonschema
+from pytz import UTC
 
 from cubicweb import (
     Binary,
+    Unauthorized,
     _,
 )
 from cubicweb.devtools.testlib import CubicWebTC
@@ -145,6 +148,57 @@ class RelationMapperTC(CubicWebTC):
             schema = adapter.creation_schema()
             self.assertIn('media_type', schema['required'])
 
+    def test_required_generic_relation(self):
+        with self.admin_access.cnx() as cnx:
+            mapper = cnx.vreg['mappers'].select(
+                'jsonschema.entity', cnx, etype='CWUser')
+            schema = mapper.jsl_document(CREATION_ROLE).get_schema()
+            self.assertIn('in_group', schema['required'])
+
+
+class InlinedRelationMapperTC(CubicWebTC):
+
+    def test_schema(self):
+        with self.admin_access.client_cnx() as cnx:
+            mapper = cnx.vreg['mappers'].select(
+                'jsonschema.relation', cnx,
+                etype='CWUser', rtype='use_email', role='subject',
+                target_types={'EmailAddress'},
+            )
+            field = mapper.jsl_field(CREATION_ROLE)
+            schema = field.get_schema()
+            expected = {
+                'title': 'use_email',
+                'type': 'array',
+                'items': {
+                    '$ref': '#/definitions/EmailAddress',
+                },
+                'description': "user's email account",
+                'definitions': {
+                    'EmailAddress': {
+                        'title': 'EmailAddress',
+                        'type': 'object',
+                        'additionalProperties': False,
+                        'properties': {
+                            'address': {
+                                'format': 'email',
+                                'maxLength': 128,
+                                'title': 'address',
+                                'type': 'string',
+                            },
+                            'alias': {
+                                'maxLength': 56,
+                                'title': 'alias',
+                                'type': 'string',
+                            },
+                        },
+                        'required': ['address'],
+                    },
+                },
+            }
+        self.maxDiff = None
+        self.assertEqual(schema, expected)
+
 
 class AttributeMappersTC(CubicWebTC):
 
@@ -171,6 +225,19 @@ class AttributeMappersTC(CubicWebTC):
             self.assertIsInstance(bin_value, Binary)
             self.assertEqual(bin_value.getvalue(), b'plop')
 
+    def test_date_value(self):
+        with self.admin_access.cnx() as cnx:
+            mapper = cnx.vreg['mappers'].select(
+                'jsonschema.relation', cnx, etype='CWUser',
+                rtype='last_login_time', role='subject',
+                target_types={'TZDatetime'})
+            instance = {'last_login_time': '2007-01-25T12:00:00Z'}
+            values = mapper.values(None, instance)
+            expected = {
+                'last_login_time': datetime(2007, 1, 25, 12, 0, tzinfo=UTC),
+            }
+            self.assertEqual(values, expected)
+
     def test_password_values(self):
         with self.admin_access.cnx() as cnx:
             users = cnx.find('CWGroup', name=u'users').one()
@@ -250,6 +317,36 @@ class AttributeMappersTC(CubicWebTC):
             }
             self.assertEqual(schema, expected)
 
+    def test_unchanged_value_not_updated(self):
+        with self.admin_access.cnx() as cnx:
+            self.create_user(cnx, u'bob')
+            cnx.commit()
+        with self.new_access(u'bob').cnx() as cnx:
+            entity = cnx.create_entity('Topic', name=u'top')
+            cnx.commit()
+            mapper = cnx.vreg['mappers'].select(
+                'jsonschema.relation', cnx, entity=entity,
+                rtype='name', role='subject', target_types={'String'})
+            # By permission, we can change the attribute.
+            values = mapper.values(entity, {'name': 'tip'})
+            self.assertEqual(values, {'name': 'tip'})
+            entity.cw_set(**values)
+            cnx.commit()
+            cnx.create_entity('Book', title=u'ti', topics=entity)
+            cnx.commit()
+            # Value change, we get something returned.
+            values = mapper.values(entity, {'name': 'top'})
+            self.assertEqual(values, {'name': 'top'})
+            # By permission, we can't change the attribute.
+            with self.assertRaises(Unauthorized):
+                entity.cw_set(**values)
+                cnx.commit()
+            cnx.rollback()
+            # But passing the actual value of attribute should not trigger
+            # entity update.
+            values = mapper.values(entity, {'name': 'tip'})
+            self.assertEqual(values, {})
+
 
 class CompoundMapperTC(CubicWebTC):
 
@@ -259,7 +356,18 @@ class CompoundMapperTC(CubicWebTC):
                 return obj
         raise ValueError(name)
 
+    def test_no_relations_attribute(self):
+        class bad_compound(CompoundMapper):
+            etype = 'Photo'
+
+        expected_regexp = 'is missing a "relations" class attribute'
+        with self.assertRaisesRegex(ValueError, expected_regexp):
+            bad_compound.__registered__(self.vreg['mappers'])
+
     def test_no_duplicate_relation_mapping(self):
+        """A CompoundMapper with the same relation 'flash' is defined in
+        test/data/mappers.py, thus this one is a duplicate and should fail.
+        """
         class bad_compound(CompoundMapper):
             etype = 'Photo'
             relations = ('flash', )
@@ -268,6 +376,33 @@ class CompoundMapperTC(CubicWebTC):
         with self.assertRaisesRegex(ValueError, expected_regexp):
             bad_compound.__registered__(self.vreg['mappers'])
 
+    def test_no_duplicate_subject_relation_mapping(self):
+        """A CompoundMapper with the same relation 'flash' is defined in
+        test/data/mappers.py, thus this one is a duplicate and should fail.
+        """
+        class bad_compound(CompoundMapper):
+            etype = 'Photo'
+            relations = (
+                ('flash', 'subject'),
+            )
+
+        expected_regexp = 'duplicate relation mapping for "Photo": flash-subject'  # noqa: E501
+        with self.assertRaisesRegex(ValueError, expected_regexp):
+            bad_compound.__registered__(self.vreg['mappers'])
+
+    def test_no_duplicate_object_relation_mapping(self):
+        class good_compound(CompoundMapper):
+            etype = 'Book'
+            relations = (
+                'author',
+                ('author', 'object'),
+            )
+
+        try:
+            good_compound.__registered__(self.vreg['mappers'])
+        except ValueError as e:
+            self.fail("unexpected {}".format(e))
+
     def test_mapped_attributes_hidden(self):
         """Make sure attributes mapped to CompoundMapper are in "hidden" uicfg
         section.
@@ -543,6 +678,57 @@ class CompoundMapperTC(CubicWebTC):
                 thumbnail = cnx.find('Thumbnail').one()
                 self.assertEqual(values['thumbnail'], [thumbnail])
 
+    def test_object_relation(self):
+
+        class my_compound(CompoundMapper):
+            name = 'My Compound'
+            title = 'My Title'
+            etype = 'Author'
+            relations = (
+                ('author', 'object'),
+            )
+
+        expected_schema = {
+            "$schema": "http://json-schema.org/draft-06/schema#",
+            "type": "object",
+            "title": u"Author",
+            "additionalProperties": False,
+            "properties": {
+                "name": {"title": u"name", "type": "string"},
+                "My Compound": {
+                    "$ref": "#/definitions/My Title",
+                },
+            },
+            "definitions": {
+                "My Title": {
+                    "type": "object",
+                    "title": u"My Title",
+                    "additionalProperties": False,
+                    "properties": {
+                        "reverse_author": {
+                            "title": u"author_object",
+                            "items": {
+                                "oneOf": [],
+                                "type": "string",
+                            },
+                            "type": "array",
+                        },
+                    },
+                },
+            },
+        }
+
+        with self.temporary_appobjects(my_compound):
+            with self.admin_access.cnx() as cnx:
+                author = cnx.create_entity(
+                    'Author', name=u'author')
+                cnx.create_entity(
+                    'Book', author=author, title=u'title')
+
+                adapter = author.cw_adapt_to('IJSONSchema')
+                schema = adapter.view_schema()
+                self.assertEqual(expected_schema, schema)
+
 
 class EntityMapperTC(CubicWebTC):
 
@@ -554,13 +740,35 @@ class EntityMapperTC(CubicWebTC):
                              ['exif_data'])
 
     def test_etypemapper_relations(self):
+        expected_rtypes = ['login', 'firstname', 'surname',
+                           'last_login_time', 'in_group', 'use_email',
+                           'picture']
+        with self.admin_access.cnx() as cnx:
+            user = cnx.find('CWUser', login=u'admin').one()
+            mapper = cnx.vreg['mappers'].select(
+                'jsonschema.entity', cnx, entity=user)
+            rtypes = [rtype
+                      for rtype, _, _ in mapper.relations(VIEW_ROLE)]
+            self.assertCountEqual(expected_rtypes, rtypes)
+            self.create_user(cnx, u'bob')
+            cnx.commit()
+        with self.new_access(u'bob').cnx() as cnx:
+            user = cnx.find('CWUser', login=u'admin').one()
+            mapper = cnx.vreg['mappers'].select(
+                'jsonschema.entity', cnx, entity=user)
+            rtypes = [rtype
+                      for rtype, _, _ in mapper.relations(VIEW_ROLE)]
+            self.assertCountEqual(expected_rtypes, rtypes)
+
+    def test_etypemapper_relations_accounts_for_uicfg(self):
         with self.admin_access.cnx() as cnx:
             mapper = cnx.vreg['mappers'].select(
-                'jsonschema.entity', cnx, etype='Thumbnail')
-            relations = [(rtype, role)
-                         for rtype, role, _ in mapper.relations('view')]
-            expected = [('data', 'subject'), ('thumbnail', 'object')]
-            self.assertCountEqual(expected, relations)
+                'jsonschema.entity', cnx, etype='CWUser')
+            fields = dict(
+                (rtype, targets)
+                for rtype, _, targets in mapper.relations(VIEW_ROLE))
+            # EmailAlias no in, per jsonschema_section.
+            self.assertEqual(fields['use_email'], {'EmailAddress'})
 
     def test_targetetypemapper_relations_and_schema(self):
         with self.admin_access.cnx() as cnx:
@@ -568,23 +776,41 @@ class EntityMapperTC(CubicWebTC):
                 'jsonschema.entity', cnx, etype='Thumbnail',
                 rtype='thumbnail', role='object',
             )
-            relations = [(rtype, role)
-                         for rtype, role, _ in mapper.relations('view')]
-            self.assertEqual([('data', 'subject')], relations)
-            schema = mapper.jsl_document(VIEW_ROLE).get_schema()
-            expected = {
-                '$schema': 'http://json-schema.org/draft-06/schema#',
-                'title': 'Thumbnail',
-                'type': 'object',
-                'properties': {
-                    'data': {
-                        'title': 'data',
-                        'type': 'string',
-                    },
+            self._check_thumbnail_relations_and_schema(mapper)
+
+    def test_targetentitymapper_relations_and_schema(self):
+        with self.admin_access.cnx() as cnx:
+            thumbnail = cnx.create_entity('Thumbnail', data=Binary(b'plip'))
+            cnx.create_entity('Photo', data=Binary(b'plop'),
+                              thumbnail=thumbnail)
+            cnx.commit()
+            mapper = cnx.vreg['mappers'].select(
+                'jsonschema.entity', cnx, entity=thumbnail,
+                rtype='thumbnail', role='object',
+            )
+            self._check_thumbnail_relations_and_schema(mapper)
+
+    def _check_thumbnail_relations_and_schema(self, mapper):
+        relations = [(rtype, role)
+                     for rtype, role, _ in mapper.relations('view')]
+        self.assertEqual([('data', 'subject')], relations)
+        relations = [(rtype, role)
+                     for rtype, role, _ in mapper.relations('creation')]
+        self.assertEqual([('data', 'subject')], relations)
+        schema = mapper.jsl_document(VIEW_ROLE).get_schema()
+        expected = {
+            '$schema': 'http://json-schema.org/draft-06/schema#',
+            'title': 'Thumbnail',
+            'type': 'object',
+            'properties': {
+                'data': {
+                    'title': 'data',
+                    'type': 'string',
                 },
-                'additionalProperties': False,
-            }
-            self.assertEqual(schema, expected)
+            },
+            'additionalProperties': False,
+        }
+        self.assertEqual(schema, expected)
 
     def test_relation_targets(self):
         with self.admin_access.cnx() as cnx:
@@ -707,7 +933,7 @@ class EntityMapperTC(CubicWebTC):
             mapper = cnx.vreg['mappers'].select(
                 'jsonschema.entity', cnx, etype='Photo')
             instance = {'data': 'plip'}
-            values = mapper.values(None, instance)
+            values = mapper.values(instance)
             self.assertEqual(list(values), ['data'])
             self.assertEqual(values['data'].getvalue(), b'plip')
             instance = {
@@ -718,7 +944,7 @@ class EntityMapperTC(CubicWebTC):
                     },
                 ],
             }
-            values = mapper.values(None, instance)
+            values = mapper.values(instance)
             self.assertCountEqual(list(values), ['data', 'thumbnail'])
             self.assertEqual(values['data'].getvalue(), b'plip')
             thumbnail, = values['thumbnail']
@@ -738,7 +964,7 @@ class EntityMapperTC(CubicWebTC):
             mapper = cnx.vreg['mappers'].select(
                 'jsonschema.entity', cnx, entity=entity)
             instance = {'data': 'plip', 'media_type': u'jpeg'}
-            values = mapper.values(entity, instance.copy())
+            values = mapper.values(instance.copy())
             # All subject relations should appear in "values"
             subjrels = [rschema.type
                         for rschema in entity.e_schema.subject_relations()
